use actix_web::{web, HttpResponse, Responder};
use rinja::Template;
use sqlx::PgPool;

use crate::{
    endpoints::{events::NewEventTemplate, IdPath},
    models::{Assignment, Event, Function, Location, Role, User},
    utils::ApplicationError,
};

#[actix_web::get("/events/{id}/edit")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let Some(event) = Event::read_by_id_including_location(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    let locations = if user.role == Role::Admin {
        Location::read_all_including_area(pool.get_ref()).await?
    } else {
        Location::read_by_area(pool.get_ref(), user.area_id).await?
    };

    let assignments = Assignment::read_all_by_event(pool.get_ref(), event.id).await?;

    let template = NewEventTemplate {
        user: user.into_inner(),
        date: event.date,
        locations,
        event: Some(event),
        amount_of_planned_posten: assignments
            .iter()
            .filter(|x| x.function == Function::Posten)
            .count(),
        is_fuehrungsassistent_planned: assignments
            .iter()
            .any(|x| x.function == Function::Fuehrungsassistent),
        is_wachhabender_planned: assignments
            .iter()
            .any(|x| x.function == Function::Wachhabender),
    };

    Ok(HttpResponse::Ok().body(template.render()?))
}
